import requests
import json

__all__ = [
    'TeleBot'
]


class TeleBot():
    """Bot that automatically sends messages over telegram

    Parameters
    ----------
    config_file : json
        json file containing the bot_token and the bot_chatID
    """

    def __init__(self, config_file) -> None:
        self._credentials = self._get_token(config_file)
        self._url = self._compose_url()

    def _get_token(self, config_file):
        """read the access token form a json file.

        Parameters
        ----------
        config_file : json
            json file containing the bot_token and the bot_chatID

        Returns
        -------
        dict
            credential info
        """
        with open(config_file, "r") as f:
            token = json.load(f)
        return token

    def _compose_url(self):
        """compose the base url to send the message

        Returns
        -------
        str
            base url
        """
        return 'https://api.telegram.org/bot' + self._credentials["bot_token"] + '/sendMessage?chat_id=' + self._credentials["bot_chatID"] + '&parse_mode=Markdown&text='

    def send_message(self, bot_message):
        """send the message over telegram

        Parameters
        ----------
        bot_message : str
            message to send

        Returns
        -------
        json
            response from the server
        """
        response = requests.get(self._url + bot_message)
        return response.json()


if __name__ == '__main__':

    # init the bot
    bot = TeleBot('.tokens/telegram.json')

    # send the message
    bot.send_message('ciao mamma')



__author__ = """Francesco Ranaudo"""
__email__ = 'francesco.ranaudo@gmail.com'
__version__ = '0.5.0'

import os

HERE = os.path.dirname(__file__)

HOME = os.path.abspath(os.path.join(HERE, '../../'))
DATA = os.path.abspath(os.path.join(HOME, 'data'))
DOCS = os.path.abspath(os.path.join(HOME, 'docs'))
TEMP = os.path.abspath(os.path.join(HOME, 'temp'))



# import paramiko
# import os
# from io import StringIO
# import sys


# def connect_with_pvtkey():
#     k = paramiko.RSAKey.from_private_key_file("/home/fr/.ssh/id_rsa", password="Sxcdews23")
#     c = paramiko.SSHClient()
#     c.set_missing_host_key_policy(paramiko.AutoAddPolicy())
#     print("connecting")
#     c.connect(hostname="192.168.0.199", username="franaudo", pkey=k)
#     print("connected")
#     # commands = [ "/home/ubuntu/firstscript.sh", "/home/ubuntu/secondscript.sh" ]
#     # for command in commands:
#     # 	print "Executing {}".format( command )
#     # 	stdin , stdout, stderr = c.exec_command(command)
#     # 	print stdout.read()
#     # 	print( "Errors")
#     # 	print stderr.read()
#     # c.close()


# # def ssh_key_gen(length=2048, type='rsa', password=None, username='jumpserver', hostname=None):
# #     """Generate user ssh private and public key

# #     Use paramiko RSAKey generate it.
# #     :return private key str and public key str
# #     """

# #     if hostname is None:
# #         hostname = os.uname()[1]

# #     f = StringIO()
# #     try:
# #         if type == 'rsa':
# #             private_key_obj = paramiko.RSAKey.generate(length)
# #         elif type == 'dsa':
# #             private_key_obj = paramiko.DSSKey.generate(length)
# #         else:
# #             raise IOError('SSH private key must be `rsa` or `dsa`')
# #         private_key_obj.write_private_key(f, password=password)
# #         private_key = f.getvalue()
# #         public_key = ssh_pubkey_gen(private_key_obj, username=username, hostname=hostname)
# #         return private_key, public_key
# #     except IOError:
# #         raise IOError('These is error when generate ssh key.')


# def create_keypair():

#     key = paramiko.RSAKey.generate(4096)
#     print(key.get_base64())  # print public key
#     key.write_private_key(sys.stdout)  # print private key

#     with open("/home/fr/.ssh/pubkey", "w") as key_file:
#         key_file.write(key.get_base64())

#     key.write_private_key("/home/fr/.ssh/pvtkey")


# if __name__ == "__main__":
#     connect_with_pvtkey()



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("CHANGELOG.rst") as history_file:
    history = history_file.read()

requirements = ["Click>=7.0", "slack_sdk", "yagmail"]

setup_requirements = ["pytest-runner", ]

test_requirements = ["pytest>=3", ]

setup(
    name="codebots",
    version="0.5.0",
    author="Francesco Ranaudo",
    author_email="ranaudo@arch.ethz.ch",
    python_requires=">=3.5",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="collection of bots for tasks automation",
    entry_points={
        "console_scripts": [
            "codebots=codebots.cli:main",
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="codebots",
    packages=["codebots"],
    package_dir={'': 'src'},
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/franaudo/codebots",
    zip_safe=False,
)



import paramiko
import os
import json

import sys
# import subprocess
# import argparse

__all__ = [
    'sshBot'
]


class sshBot():
    """sshBot to help with ssh connections to a server.

    Parameters
    ----------
    hostname : str
        ip address of the server, by default None.
    username : str
        username on the server, by default None.
    password : str
        password on the server, by default None.
    pvtkey : str
        path to the private RSA key file, by default None.

    Attributes
    ----------
    hostname : str
        ip address of the server, by default None.
    username : str
        username on the server, by default None.
    password : str
        password on the server, by default None.
    pvtkey : str
        path to the private RSA key file, by default None.
    ssh_client : obj
        paramiko `SSHClient` object, if a connection is active, otherwise None.
    sftp_client : obj
        paramiko `SFTPClient` object, if a connection is active, otherwise None.
    """

    def __init__(self, hostname=None, username=None, password=None, pvtkey=None) -> None:

        self.hostname = hostname
        self.username = username
        self.password = password
        self.pvtkey = pvtkey
        self.ssh_client = None
        self.sftp_client = None

    @classmethod
    def from_credentials_file(cls, json_file):
        """Get credentials from json file

        Parameters
        ----------
        json_file : str
            path to the json file with the credentials for the server.
            it should have the following structure:

            .. code-block:: json

                {
                    "hostname" : "_______",
                    "username" : "_______",
                    "password" : "_______",
                    "pvtkey"   : "_______"
                }

        Returns
        -------
        cls
            sshBot
        """
        with open(json_file, 'r') as f:
            credentials = json.load(f)
        return cls(**credentials)

    def gen_keypair(self, ssh_folder):
        """Create a set of public and private keys and save them in the given
        folder.

        Parameters
        ----------
        ssh_folder : str
            folder where the keys are saved
        """
        key = paramiko.RSAKey.generate(4096)
        with open(os.path.join(ssh_folder, "pubkey"), "w") as key_file:
            key_file.write(key.get_base64())
        with open(os.path.join(ssh_folder, "pvtkey"), "w") as key_file:
            key_file.write(key.write_private_key(sys.stdout))

    def connect_ssh_client(self):
        """Establish the ssh connection

        Returns
        -------
        obj
            ssh_client obj
        """
        ssh_client = paramiko.SSHClient()
        ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        if self.pvtkey:
            k = paramiko.RSAKey.from_private_key_file(self.pvtkey, password=self.password)
        ssh_client.connect(hostname=self.hostname, username=self.username, password=self.password, pkey=k)
        print("connected")
        return ssh_client

    def execute_cmds(self, commands, close_connection=True):
        """Execute general command on the server side

        Parameters
        ----------
        commands : list of str
            list of commands (str) to execute on the server.
        close_connection : bool (optional)
            if true close the ssh connection, by default True. Leave the connection
            open if you plan to run several commands.

        Returns
        -------
        None
        """
        if not self.ssh_client:
            self.ssh_client = self.connect_ssh_client()

        # commands = [ "/home/ubuntu/firstscript.sh", "/home/ubuntu/secondscript.sh" ]
        for command in commands:
            print("Executing {}".format(command))
            stdin, stdout, stderr = self.ssh_client.exec_command(command)
            print(stdout.read())
            print("Errors")
            print(stderr.read())

        # close the connection
        if close_connection:
            self.ssh_client.close()
            self.ssh_client = None

    def connect_sftp_client(self):
        """Connect to the server through SFPT on port 22.

        Returns
        -------
        obj
            paramiko sfpt client object
        """
        transport = paramiko.Transport((self.hostname, 22))
        if self.pvtkey:
            k = paramiko.RSAKey.from_private_key_file(self.pvtkey, password=self.password)
        transport.connect(username=self.username, password=self.password, pkey=k)
        return paramiko.SFTPClient.from_transport(transport)

    def get_folder_from_server(self, path, dest, recursive=True, close_connection=True):
        """Retrieve the content of a folder from the server.

        Parameters
        ----------
        sftp_client : obj
            paramiko sftp client object
        path : str
            path to the folder on the server
        dest : str
            path to the folder on the client (local)
        recursive : bool (optional)
            if true get subfolders content, by default
        close_connection : bool (optional)
            if true close the ssh connection, by default True. Leave the connection open if you plan to run several commands.

        Warnings
        --------
        If in any folder there are files without extensions, the code will fail!
        """

        # connect to the server through SFPT
        if not self.sftp_client:
            self.sftp_client = self.connect_sftp_client()
            print("connection enstablished")

        for item in self.sftp_client.listdir(path):
            remotefile = os.path.join(path, str(item))
            if not os.path.isdir(dest):
                os.mkdir(dest)
            localfilepath = os.path.join(dest, str(item))
            # check if it is a folder (note file w/o ext will thorw an exception!)
            if '.' in item:
                self.sftp_client.get(remotefile, localfilepath)
            else:
                if recursive:
                    self.get_folder_from_server(self.sftp_client, remotefile, localfilepath)
        print("files tranferred!")
        # close the connection
        if close_connection:
            self.sftp_client.close()
            self.sftp_client = None
            print("connection closed")


if __name__ == '__main__':

    bot = sshBot.from_credentials_file(".tokens/home.json")

    bot.execute_cmds(commands=['ls'], close_connection=False)
    print(bot.ssh_client)
    bot.execute_cmds(commands=['ls -l'], close_connection=True)
    print(bot.ssh_client)



# import time
# import schedule
# def report():
#     my_balance = 10   ## Replace this number with an API call to fetch your account balance
#     my_message = "Current balance is: {}".format(my_balance)   ## Customize your message
#     telegram_bot_sendtext(my_message)
# schedule.every().day.at("12:00").do(report)
# while True:
#     schedule.run_pending()
#     time.sleep(1)
